@@ -43,7 +43,6 @@ typedef struct
 #define ENABLE_SCRIPTS_SETTING "enable-javascript"
 #define DEFAULT_ENCODING_SETTING "default-charset"
 static WebKitSettings *webkit_settings = NULL;
-static WebKitWebViewGroup *web_view_group = NULL;
 
 static void
 user_style_sheet_output_stream_splice_cb (GOutputStream *output_stream,
@@ -54,9 +53,14 @@ user_style_sheet_output_stream_splice_cb (GOutputStream *output_stream,
 
   bytes = g_output_stream_splice_finish (output_stream, result, NULL);
   if (bytes > 0) {
-    webkit_web_view_group_add_user_style_sheet (web_view_group,
-                                                g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (output_stream)),
-                                                NULL, NULL, NULL, WEBKIT_INJECTED_CONTENT_FRAMES_ALL);
+    WebKitUserStyleSheet *style_sheet;
+
+    style_sheet = webkit_user_style_sheet_new (g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (output_stream)),
+                                               WEBKIT_USER_CONTENT_INJECT_ALL_FRAMES, WEBKIT_USER_STYLE_LEVEL_USER,
+                                               NULL, NULL);
+    webkit_user_content_manager_add_style_sheet (WEBKIT_USER_CONTENT_MANAGER (ephy_embed_shell_get_user_content_manager (ephy_embed_shell_get_default ())),
+                                                 style_sheet);
+    webkit_user_style_sheet_unref (style_sheet);
   }
 }
 
@@ -94,7 +98,7 @@ webkit_pref_callback_user_stylesheet (GSettings *settings,
   value = g_settings_get_boolean (settings, key);
 
   if (!value)
-    webkit_web_view_group_remove_all_user_style_sheets (web_view_group);
+    webkit_user_content_manager_remove_all_style_sheets (WEBKIT_USER_CONTENT_MANAGER (ephy_embed_shell_get_user_content_manager (ephy_embed_shell_get_default ())));
   else {
     GFile *file;
     char *filename;
@@ -241,7 +245,7 @@ webkit_pref_callback_font_size (GSettings *settings,
   int size = 9; /* FIXME: What to use here? */
 
   char *schema = NULL;
-  g_object_get (settings, "schema", &schema, NULL);
+  g_object_get (settings, "schema-id", &schema, NULL);
 
   /* If we are changing a GNOME font value and we are not using GNOME fonts in
    * Epiphany, return. */
@@ -278,7 +282,7 @@ webkit_pref_callback_font_family (GSettings *settings,
   char *value = NULL;
 
   char *schema = NULL;
-  g_object_get (settings, "schema", &schema, NULL);
+  g_object_get (settings, "schema-id", &schema, NULL);
 
   /* If we are changing a GNOME font value and we are not using GNOME fonts in
    * Epiphany, return. */
@@ -315,6 +319,7 @@ webkit_pref_callback_accept_languages (GSettings *settings,
   GArray *array;
   char **languages;
   int i;
+  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 
   languages = g_settings_get_strv (settings, key);
 
@@ -331,7 +336,7 @@ webkit_pref_callback_accept_languages (GSettings *settings,
 
   ephy_langs_sanitise (array);
 
-  webkit_web_context_set_preferred_languages (webkit_web_context_get_default (),
+  webkit_web_context_set_preferred_languages (ephy_embed_shell_get_web_context (shell),
                                               (const char * const *)array->data);
 
   g_strfreev (languages);
@@ -366,12 +371,14 @@ webkit_pref_callback_cookie_accept_policy (GSettings *settings,
 {
   WebKitCookieManager *cookie_manager;
   char *value;
+  EphyEmbedShell *shell;
 
   value = g_settings_get_string (settings, key);
   if (!value)
     return;
 
-  cookie_manager = webkit_web_context_get_cookie_manager (webkit_web_context_get_default ());
+  shell = ephy_embed_shell_get_default ();
+  cookie_manager = webkit_web_context_get_cookie_manager (ephy_embed_shell_get_web_context (shell));
   ephy_embed_prefs_set_cookie_accept_policy (cookie_manager, value);
   g_free (value);
 }
@@ -442,10 +449,11 @@ webkit_pref_callback_enable_spell_checking (GSettings *settings,
                                             char *key,
                                             gpointer data)
 {
-  WebKitWebContext *web_context = NULL;
+  WebKitWebContext *web_context;
   gboolean value = FALSE;
   char **languages = NULL;
   char *langs = NULL;
+  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 
   value = g_settings_get_boolean (settings, key);
 
@@ -460,7 +468,7 @@ webkit_pref_callback_enable_spell_checking (GSettings *settings,
     langs = g_strjoinv (",", languages);
   }
 
-  web_context = webkit_web_context_get_default ();
+  web_context = ephy_embed_shell_get_web_context (shell);
   webkit_web_context_set_spell_checking_enabled (web_context, value);
   webkit_web_context_set_spell_checking_languages (web_context, (const char* const *)languages);
 
@@ -529,15 +537,12 @@ ephy_embed_prefs_init (gpointer user_data)
 {
   int i;
 
-  web_view_group = webkit_web_view_group_new ("Ephy WebView Group");
-  webkit_settings = webkit_web_view_group_get_settings (web_view_group);
-
-  g_object_set (webkit_settings,
-                "enable-developer-extras", TRUE,
-                "enable-fullscreen", TRUE,
-                "enable-site-specific-quirks", TRUE,
-                "enable-dns-prefetching", TRUE,
-                NULL);
+  webkit_settings = webkit_settings_new_with_settings ("enable-developer-extras", TRUE,
+                                                       "enable-fullscreen", TRUE,
+                                                       "enable-site-specific-quirks", TRUE,
+                                                       "enable-dns-prefetching", TRUE,
+                                                       "javascript-can-open-windows-automatically", TRUE,
+                                                       NULL);
 
   for (i = 0; i < G_N_ELEMENTS (webkit_pref_entries); i++) {
     GSettings *settings;
@@ -564,10 +569,6 @@ ephy_embed_prefs_init (gpointer user_data)
                    EPHY_PREFS_ENABLE_CARET_BROWSING,
                    webkit_settings, "enable-caret-browsing",
                    G_SETTINGS_BIND_GET);
-  g_settings_bind (EPHY_SETTINGS_WEB,
-                   EPHY_PREFS_WEB_ENABLE_POPUPS,
-                   webkit_settings, "javascript-can-open-windows-automatically",
-                   G_SETTINGS_BIND_GET);
   g_settings_bind (EPHY_SETTINGS_WEB,
                    EPHY_PREFS_WEB_ENABLE_PLUGINS,
                    webkit_settings, "enable-plugins",
@@ -592,12 +593,11 @@ ephy_embed_prefs_init (gpointer user_data)
                    EPHY_PREFS_ENABLE_SMOOTH_SCROLLING,
                    webkit_settings, "enable-smooth-scrolling",
                    G_SETTINGS_BIND_GET);
-
-  return web_view_group;
+  return webkit_settings;
 }
 
-WebKitWebViewGroup *
-ephy_embed_prefs_get_web_view_group (void)
+WebKitSettings *
+ephy_embed_prefs_get_settings (void)
 {
   static GOnce once_init = G_ONCE_INIT;
 
