@@ -34,6 +34,7 @@
 #include "ephy-profile-utils.h"
 #include "ephy-settings.h"
 #include "ephy-snapshot-service.h"
+#include "ephy-web-app-utils.h"
 #include "ephy-web-extension-proxy.h"
 #include "ephy-web-extension-names.h"
 
@@ -49,6 +50,7 @@
 
 struct _EphyEmbedShellPrivate
 {
+  WebKitWebContext *web_context;
   EphyHistoryService *global_history_service;
   EphyEncodings *encodings;
   GtkPageSetup *page_setup;
@@ -61,9 +63,6 @@ struct _EphyEmbedShellPrivate
   GDBusConnection *bus;
   GList *web_extensions;
   guint web_extensions_page_created_signal_id;
-  guint web_extensions_form_auth_save_signal_id;
-  guint web_extensions_remove_from_overview_signal_id;
-  guint web_extensions_allow_tls_certificate_signal_id;
 };
 
 enum
@@ -72,6 +71,8 @@ enum
   RESTORED_WINDOW,
   WEB_VIEW_CREATED,
   PAGE_CREATED,
+  ALLOW_TLS_CERTIFICATE,
+  FORM_AUTH_DATA_SAVE_REQUESTED,
 
   LAST_SIGNAL
 };
@@ -107,6 +108,7 @@ ephy_embed_shell_dispose (GObject *object)
   g_clear_object (&priv->global_history_service);
   g_clear_object (&priv->about_handler);
   g_clear_object (&priv->user_content);
+  g_clear_object (&priv->web_context);
 
   G_OBJECT_CLASS (ephy_embed_shell_parent_class)->dispose (object);
 }
@@ -133,25 +135,25 @@ ephy_embed_shell_find_web_extension (EphyEmbedShell *shell,
 }
 
 static void
-web_extension_form_auth_save_requested (GDBusConnection *connection,
-                                        const char *sender_name,
-                                        const char *object_path,
-                                        const char *interface_name,
-                                        const char *signal_name,
-                                        GVariant *parameters,
-                                        EphyEmbedShell *shell)
+web_extension_form_auth_data_message_received_cb (WebKitUserContentManager *manager,
+                                                  WebKitJavascriptResult *message,
+                                                  EphyEmbedShell *shell)
 {
-  EphyWebExtensionProxy *web_extension;
   guint request_id;
   guint64 page_id;
   const char *hostname;
   const char *username;
+  GVariant *variant;
+  gchar *message_str;
 
-  g_variant_get (parameters, "(ut&s&s)", &request_id, &page_id, &hostname, &username);
-  web_extension = ephy_embed_shell_find_web_extension (shell, sender_name);
-  if (!web_extension)
-    return;
-  ephy_web_extension_proxy_form_auth_save_requested (web_extension, request_id, page_id, hostname, username);
+  message_str = ephy_embed_utils_get_js_result_as_string (message);
+  variant = g_variant_parse (G_VARIANT_TYPE ("(utss)"), message_str, NULL, NULL, NULL);
+  g_free (message_str);
+
+  g_variant_get (variant, "(ut&s&s)", &request_id, &page_id, &hostname, &username);
+  g_signal_emit (shell, signals[FORM_AUTH_DATA_SAVE_REQUESTED], 0,
+                 request_id, page_id, hostname, username);
+  g_variant_unref (variant);
 }
 
 static void
@@ -246,23 +248,20 @@ history_set_url_hidden_cb (EphyHistoryService *service,
 }
 
 static void
-web_extension_remove_from_overview (GDBusConnection *connection,
-                                    const char *sender_name,
-                                    const char *object_path,
-                                    const char *interface_name,
-                                    const char *signal_name,
-                                    GVariant *parameters,
-                                    EphyEmbedShell *shell)
+web_extension_overview_message_received_cb (WebKitUserContentManager *manager,
+                                            WebKitJavascriptResult *message,
+                                            EphyEmbedShell *shell)
 {
-  const char *url_to_remove;
+  char *url_to_remove;
 
-  g_variant_get (parameters, "(&s)", &url_to_remove);
+  url_to_remove = ephy_embed_utils_get_js_result_as_string (message);
 
   shell->priv->hiding_overview_item++;
   ephy_history_service_set_url_hidden (shell->priv->global_history_service,
                                        url_to_remove, TRUE, NULL,
                                        (EphyHistoryJobCallback) history_set_url_hidden_cb,
                                        shell);
+  g_free (url_to_remove);
 
   if (shell->priv->update_overview_timeout_id > 0)
     g_source_remove (shell->priv->update_overview_timeout_id);
@@ -273,23 +272,26 @@ web_extension_remove_from_overview (GDBusConnection *connection,
 }
 
 static void
-web_extension_allow_tls_certificate (GDBusConnection *connection,
-                                     const char *sender_name,
-                                     const char *object_path,
-                                     const char *interface_name,
-                                     const char *signal_name,
-                                     GVariant *parameters,
-                                     EphyEmbedShell *shell)
+web_extension_tls_error_page_message_received_cb (WebKitUserContentManager *manager,
+                                                  WebKitJavascriptResult *message,
+                                                  EphyEmbedShell *shell)
 {
-  EphyWebExtensionProxy *web_extension;
   guint64 page_id;
 
-  web_extension = ephy_embed_shell_find_web_extension (shell, sender_name);
-  if (!web_extension)
-    return;
+  page_id = ephy_embed_utils_get_js_result_as_number (message);
+  g_signal_emit (shell, signals[ALLOW_TLS_CERTIFICATE], 0, page_id);
+}
 
-  g_variant_get (parameters, "(t)", &page_id);
-  ephy_web_extension_proxy_allow_tls_certificate (web_extension, page_id);
+static void
+web_extension_about_apps_message_received_cb (WebKitUserContentManager *manager,
+                                              WebKitJavascriptResult *message,
+                                              EphyEmbedShell *shell)
+{
+  char *app_id;
+
+  app_id = ephy_embed_utils_get_js_result_as_string (message);
+  ephy_web_application_delete (app_id);
+  g_free (app_id);
 }
 
 static void
@@ -545,44 +547,10 @@ ephy_embed_shell_setup_web_extensions_connection (EphyEmbedShell *shell)
                                         (GDBusSignalCallback)web_extension_page_created,
                                         shell,
                                         NULL);
-  shell->priv->web_extensions_form_auth_save_signal_id =
-    g_dbus_connection_signal_subscribe (shell->priv->bus,
-                                        NULL,
-                                        EPHY_WEB_EXTENSION_INTERFACE,
-                                        "FormAuthDataSaveConfirmationRequired",
-                                        EPHY_WEB_EXTENSION_OBJECT_PATH,
-                                        NULL,
-                                        G_DBUS_SIGNAL_FLAGS_NONE,
-                                        (GDBusSignalCallback)web_extension_form_auth_save_requested,
-                                        shell,
-                                        NULL);
-  shell->priv->web_extensions_remove_from_overview_signal_id =
-    g_dbus_connection_signal_subscribe (shell->priv->bus,
-                                        NULL,
-                                        EPHY_WEB_EXTENSION_INTERFACE,
-                                        "RemoveItemFromOverview",
-                                        EPHY_WEB_EXTENSION_OBJECT_PATH,
-                                        NULL,
-                                        G_DBUS_SIGNAL_FLAGS_NONE,
-                                        (GDBusSignalCallback)web_extension_remove_from_overview,
-                                        shell,
-                                        NULL);
-  shell->priv->web_extensions_allow_tls_certificate_signal_id =
-    g_dbus_connection_signal_subscribe (shell->priv->bus,
-                                        NULL,
-                                        EPHY_WEB_EXTENSION_INTERFACE,
-                                        "AllowTLSCertificate",
-                                        EPHY_WEB_EXTENSION_OBJECT_PATH,
-                                        NULL,
-                                        G_DBUS_SIGNAL_FLAGS_NONE,
-                                        (GDBusSignalCallback)web_extension_allow_tls_certificate,
-                                        shell,
-                                        NULL);
 }
 
 static void
-ephy_embed_shell_setup_process_model (EphyEmbedShell *shell,
-                                      WebKitWebContext *web_context)
+ephy_embed_shell_setup_process_model (EphyEmbedShell *shell)
 {
   EphyPrefsProcessModel process_model;
 
@@ -593,63 +561,118 @@ ephy_embed_shell_setup_process_model (EphyEmbedShell *shell,
 
   switch (process_model) {
   case EPHY_PREFS_PROCESS_MODEL_SHARED_SECONDARY_PROCESS:
-    webkit_web_context_set_process_model (web_context, WEBKIT_PROCESS_MODEL_SHARED_SECONDARY_PROCESS);
+    webkit_web_context_set_process_model (shell->priv->web_context,
+                                          WEBKIT_PROCESS_MODEL_SHARED_SECONDARY_PROCESS);
     break;
   case EPHY_PREFS_PROCESS_MODEL_ONE_SECONDARY_PROCESS_PER_WEB_VIEW:
-    webkit_web_context_set_process_model (web_context, WEBKIT_PROCESS_MODEL_MULTIPLE_SECONDARY_PROCESSES);
+    webkit_web_context_set_process_model (shell->priv->web_context,
+                                          WEBKIT_PROCESS_MODEL_MULTIPLE_SECONDARY_PROCESSES);
     break;
   }
 }
 
+static void
+ephy_embed_shell_create_web_context (EphyEmbedShell *embed_shell)
+{
+  char *local_storage_path;
+  EphyEmbedShellPrivate *priv = embed_shell->priv;
+
+  /* Local Storage */
+  local_storage_path = g_build_filename (EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE (priv->mode) ?
+                                         ephy_dot_dir () : g_get_user_data_dir (),
+                                         g_get_prgname (), "localstorage", NULL);
+  priv->web_context = g_object_new (WEBKIT_TYPE_WEB_CONTEXT, "local-storage-directory", local_storage_path, NULL);
+  g_free (local_storage_path);
+}
+
 static void
 ephy_embed_shell_startup (GApplication* application)
 {
   EphyEmbedShell *shell = EPHY_EMBED_SHELL (application);
-  EphyEmbedShellMode mode;
-  char *disk_cache_dir;
-  WebKitWebContext *web_context;
+  EphyEmbedShellPrivate *priv = shell->priv;
+  char *base_cache_dir;
+  char *favicon_db_path;
   WebKitCookieManager *cookie_manager;
   char *filename;
   char *cookie_policy;
 
   G_APPLICATION_CLASS (ephy_embed_shell_parent_class)->startup (application);
 
-  /* We're not remoting, setup the Web Context. */
-  mode = shell->priv->mode;
+  /* We're not remoting, setup the Web Context if we are not running in a test.
+     Tests already do this after construction. */
+  if (priv->mode != EPHY_EMBED_SHELL_MODE_TEST)
+    ephy_embed_shell_create_web_context (embed_shell);
 
   ephy_embed_shell_setup_web_extensions_connection (shell);
 
-  web_context = webkit_web_context_get_default ();
-  ephy_embed_shell_setup_process_model (shell, web_context);
-  g_signal_connect (web_context, "initialize-web-extensions",
+  /* User content manager */
+  if (priv->mode != EPHY_EMBED_SHELL_MODE_TEST)
+    shell->priv->user_content = webkit_user_content_manager_new ();
+
+  webkit_user_content_manager_register_script_message_handler (shell->priv->user_content,
+                                                               "overview");
+  g_signal_connect (shell->priv->user_content, "script-message-received::overview",
+                    G_CALLBACK (web_extension_overview_message_received_cb),
+                    shell);
+
+  webkit_user_content_manager_register_script_message_handler (shell->priv->user_content,
+                                                               "tlsErrorPage");
+  g_signal_connect (shell->priv->user_content, "script-message-received::tlsErrorPage",
+                    G_CALLBACK (web_extension_tls_error_page_message_received_cb),
+                    shell);
+
+  webkit_user_content_manager_register_script_message_handler (shell->priv->user_content,
+                                                               "formAuthData");
+  g_signal_connect (shell->priv->user_content, "script-message-received::formAuthData",
+                    G_CALLBACK (web_extension_form_auth_data_message_received_cb),
+                    shell);
+
+  webkit_user_content_manager_register_script_message_handler (shell->priv->user_content,
+                                                               "aboutApps");
+  g_signal_connect (shell->priv->user_content, "script-message-received::aboutApps",
+                    G_CALLBACK (web_extension_about_apps_message_received_cb),
+                    shell);
+
+  ephy_embed_shell_setup_process_model (shell);
+  g_signal_connect (priv->web_context, "initialize-web-extensions",
                     G_CALLBACK (initialize_web_extensions),
                     shell);
 
   /* Disk Cache */
-  disk_cache_dir = g_build_filename (EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE (mode) ?
+  base_cache_dir = g_build_filename (EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE (priv->mode) ?
                                      ephy_dot_dir () : g_get_user_cache_dir (),
                                      g_get_prgname (), NULL);
-  webkit_web_context_set_disk_cache_directory (web_context, disk_cache_dir);
-  g_free (disk_cache_dir);
+  webkit_web_context_set_disk_cache_directory (priv->web_context, base_cache_dir);
+
+  /* Favicon Database */
+  favicon_db_path = g_build_filename (base_cache_dir, "icondatabase", NULL);
+  webkit_web_context_set_favicon_database_directory (priv->web_context, favicon_db_path);
+  g_free (favicon_db_path);
+
+  g_free (base_cache_dir);
+
+  /* Do not ignore TLS errors. */
+  webkit_web_context_set_tls_errors_policy (priv->web_context, WEBKIT_TLS_ERRORS_POLICY_FAIL);
+
 
   /* about: URIs handler */
   shell->priv->about_handler = ephy_about_handler_new ();
-  webkit_web_context_register_uri_scheme (web_context,
+  webkit_web_context_register_uri_scheme (priv->web_context,
                                           EPHY_ABOUT_SCHEME,
                                           (WebKitURISchemeRequestCallback)about_request_cb,
                                           shell, NULL);
 
   /* Register about scheme as local so that it can contain file resources */
-  webkit_security_manager_register_uri_scheme_as_local (webkit_web_context_get_security_manager (web_context),
+  webkit_security_manager_register_uri_scheme_as_local (webkit_web_context_get_security_manager (priv->web_context),
                                                         EPHY_ABOUT_SCHEME);
 
   /* ephy-resource handler */
-  webkit_web_context_register_uri_scheme (web_context, "ephy-resource",
+  webkit_web_context_register_uri_scheme (priv->web_context, "ephy-resource",
                                           (WebKitURISchemeRequestCallback)ephy_resource_request_cb,
                                           NULL, NULL);
 
   /* Store cookies in moz-compatible SQLite format */
-  cookie_manager = webkit_web_context_get_cookie_manager (web_context);
+  cookie_manager = webkit_web_context_get_cookie_manager (priv->web_context);
   filename = g_build_filename (ephy_dot_dir (), "cookies.sqlite", NULL);
   webkit_cookie_manager_set_persistent_storage (cookie_manager, filename,
                                                 WEBKIT_COOKIE_PERSISTENT_STORAGE_SQLITE);
@@ -668,26 +691,16 @@ ephy_embed_shell_shutdown (GApplication* application)
 
   G_APPLICATION_CLASS (ephy_embed_shell_parent_class)->shutdown (application);
 
+  webkit_user_content_manager_unregister_script_message_handler (priv->user_content, "overview");
+  webkit_user_content_manager_unregister_script_message_handler (priv->user_content, "tlsErrorPage");
+  webkit_user_content_manager_unregister_script_message_handler (priv->user_content, "formAuthData");
+  webkit_user_content_manager_unregister_script_message_handler (priv->user_content, "aboutApps");
+
   if (priv->web_extensions_page_created_signal_id > 0) {
     g_dbus_connection_signal_unsubscribe (priv->bus, priv->web_extensions_page_created_signal_id);
     priv->web_extensions_page_created_signal_id = 0;
   }
 
-  if (priv->web_extensions_form_auth_save_signal_id > 0) {
-    g_dbus_connection_signal_unsubscribe (priv->bus, priv->web_extensions_form_auth_save_signal_id);
-    priv->web_extensions_form_auth_save_signal_id = 0;
-  }
-
-  if (priv->web_extensions_remove_from_overview_signal_id > 0) {
-    g_dbus_connection_signal_unsubscribe (priv->bus, priv->web_extensions_remove_from_overview_signal_id);
-    priv->web_extensions_remove_from_overview_signal_id = 0;
-  }
-
-  if (priv->web_extensions_allow_tls_certificate_signal_id > 0) {
-    g_dbus_connection_signal_unsubscribe (priv->bus, priv->web_extensions_allow_tls_certificate_signal_id);
-    priv->web_extensions_allow_tls_certificate_signal_id = 0;
-  }
-
   g_list_foreach (priv->web_extensions, (GFunc)ephy_embed_shell_unwatch_web_extension, application);
 
   g_object_unref (ephy_embed_prefs_get_settings ());
@@ -728,6 +741,22 @@ ephy_embed_shell_get_property (GObject *object,
   }
 }
 
+static void
+ephy_embed_shell_constructed (GObject *object)
+{
+  EphyEmbedShell *embed_shell;
+
+  G_OBJECT_CLASS (ephy_embed_shell_parent_class)->constructed (object);
+
+  embed_shell = EPHY_EMBED_SHELL (object);
+  /* Tests do not run the EmbedShell application instance, so make sure that
+     there is a web context and a user content manager for them. */
+  if (ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_TEST) {
+    ephy_embed_shell_create_web_context (embed_shell);
+    embed_shell->priv->user_content = webkit_user_content_manager_new ();
+  }
+}
+
 static void
 ephy_embed_shell_init (EphyEmbedShell *shell)
 {
@@ -747,6 +776,7 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
   object_class->dispose = ephy_embed_shell_dispose;
   object_class->set_property = ephy_embed_shell_set_property;
   object_class->get_property = ephy_embed_shell_get_property;
+  object_class->constructed = ephy_embed_shell_constructed;
 
   application_class->startup = ephy_embed_shell_startup;
   application_class->shutdown = ephy_embed_shell_shutdown;
@@ -834,6 +864,47 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
                   G_TYPE_UINT64,
                   EPHY_TYPE_WEB_EXTENSION_PROXY);
 
+  /**
+   * EphyEmbedShell::allow-tls-certificate:
+   * @shell: the #EphyEmbedShell
+   * @page_id: the identifier of the web page
+   *
+   * Emitted when the web extension requests an exception be
+   * permitted for the invalid TLS certificate on the given page
+   */
+  signals[ALLOW_TLS_CERTIFICATE] =
+    g_signal_new ("allow-tls-certificate",
+                  EPHY_TYPE_EMBED_SHELL,
+                  G_SIGNAL_RUN_FIRST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_generic,
+                  G_TYPE_NONE, 1,
+                  G_TYPE_UINT64);
+
+  /**
+   * EphyEmbedShell::form-auth-data-save-requested:
+   * @shell: the #EphyEmbedShell
+   * @request_id: the identifier of the request
+   * @page_id: the identifier of the web page
+   * @hostname: the hostname
+   * @username: the username
+   *
+   * Emitted when a web page requests confirmation to save
+   * the form authentication data for the given @hostname and
+   * @username
+   */
+  signals[FORM_AUTH_DATA_SAVE_REQUESTED] =
+    g_signal_new ("form-auth-data-save-requested",
+                  EPHY_TYPE_EMBED_SHELL,
+                  G_SIGNAL_RUN_FIRST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_generic,
+                  G_TYPE_NONE, 4,
+                  G_TYPE_UINT,
+                  G_TYPE_UINT64,
+                  G_TYPE_STRING,
+                  G_TYPE_STRING);
+
   g_type_class_add_private (object_class, sizeof (EphyEmbedShellPrivate));
 }
 
@@ -1040,14 +1111,17 @@ ephy_embed_shell_launch_handler (EphyEmbedShell *shell,
 void
 ephy_embed_shell_clear_cache (EphyEmbedShell *shell)
 {
-  webkit_web_context_clear_cache (webkit_web_context_get_default ());
+  webkit_web_context_clear_cache (shell->priv->web_context);
 }
 
 WebKitUserContentManager *
 ephy_embed_shell_get_user_content_manager (EphyEmbedShell *shell)
 {
-  if (!shell->priv->user_content)
-    shell->priv->user_content = webkit_user_content_manager_new ();
-
   return shell->priv->user_content;
 }
+
+WebKitWebContext *
+ephy_embed_shell_get_web_context (EphyEmbedShell *shell)
+{
+  return shell->priv->web_context;
+}
