@@ -28,19 +28,10 @@ struct _EphyWebExtensionProxyPrivate
 {
   GDBusProxy *proxy;
   gchar *name_owner;
+  GCancellable *cancellable;
   guint watch_name_id;
-  guint form_auth_save_signal_id;
 };
 
-enum
-{
-  FORM_AUTH_DATA_SAVE_REQUESTED,
-
-  LAST_SIGNAL
-};
-
-static guint signals[LAST_SIGNAL];
-
 G_DEFINE_TYPE (EphyWebExtensionProxy, ephy_web_extension_proxy, G_TYPE_OBJECT)
 
 static void
@@ -58,6 +49,8 @@ ephy_web_extension_proxy_dispose (GObject *object)
 {
   EphyWebExtensionProxyPrivate *priv = EPHY_WEB_EXTENSION_PROXY (object)->priv;
 
+  g_clear_object (&priv->cancellable);
+
   if (priv->watch_name_id > 0) {
     g_bus_unwatch_name (priv->watch_name_id);
     priv->watch_name_id = 0;
@@ -82,30 +75,6 @@ ephy_web_extension_proxy_class_init (EphyWebExtensionProxyClass *klass)
   object_class->finalize = ephy_web_extension_proxy_finalize;
   object_class->dispose = ephy_web_extension_proxy_dispose;
 
-  /**
-   * EphyWebExtensionProxy::form-auth-data-save-requested:
-   * @web_extension: the #EphyWebExtensionProxy
-   * @request_id: the identifier of the request
-   * @page_id: the identifier of the web page
-   * @hostname: the hostname
-   * @username: the username
-   *
-   * Emitted when a web page requests confirmation to save
-   * the form authentication data for the given @hostname and
-   * @username
-   **/
-  signals[FORM_AUTH_DATA_SAVE_REQUESTED] =
-    g_signal_new ("form-auth-data-save-requested",
-                  EPHY_TYPE_WEB_EXTENSION_PROXY,
-                  G_SIGNAL_RUN_FIRST,
-                  0, NULL, NULL,
-                  g_cclosure_marshal_generic,
-                  G_TYPE_NONE, 4,
-                  G_TYPE_UINT,
-                  G_TYPE_UINT64,
-                  G_TYPE_STRING,
-                  G_TYPE_STRING);
-
   g_type_class_add_private (object_class, sizeof (EphyWebExtensionProxyPrivate));
 }
 
@@ -130,6 +99,7 @@ web_extension_appeared_cb (GDBusConnection *connection,
                            EphyWebExtensionProxy *web_extension)
 {
   web_extension->priv->name_owner = g_strdup (name_owner);
+  web_extension->priv->cancellable = g_cancellable_new ();
   g_dbus_proxy_new (connection,
                     G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START |
                     G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES |
@@ -138,11 +108,20 @@ web_extension_appeared_cb (GDBusConnection *connection,
                     name,
                     EPHY_WEB_EXTENSION_OBJECT_PATH,
                     EPHY_WEB_EXTENSION_INTERFACE,
-                    NULL,
+                    web_extension->priv->cancellable,
                     (GAsyncReadyCallback)web_extension_proxy_created_cb,
                     web_extension);
 }
 
+static void
+web_extension_vanished_cb (GDBusConnection *connection,
+                           const gchar *name,
+                           EphyWebExtensionProxy *web_extension)
+{
+  if (web_extension->priv->name_owner)
+    g_object_unref (web_extension);
+}
+
 static void
 ephy_web_extension_proxy_watch_name (EphyWebExtensionProxy *web_extension,
                                      GDBusConnection* bus,
@@ -155,9 +134,9 @@ ephy_web_extension_proxy_watch_name (EphyWebExtensionProxy *web_extension,
                                     service_name,
                                     G_BUS_NAME_WATCHER_FLAGS_NONE,
                                     (GBusNameAppearedCallback)web_extension_appeared_cb,
-                                    NULL,
+                                    (GBusNameVanishedCallback)web_extension_vanished_cb,
                                     web_extension,
-                                    (GDestroyNotify)g_object_unref);
+                                    NULL);
 }
 
 EphyWebExtensionProxy *
@@ -183,19 +162,6 @@ ephy_web_extension_proxy_get_name_owner (EphyWebExtensionProxy *web_extension)
   return web_extension->priv->name_owner;
 }
 
-void
-ephy_web_extension_proxy_form_auth_save_requested (EphyWebExtensionProxy *web_extension,
-                                                   guint request_id,
-                                                   guint64 page_id,
-                                                   const char *hostname,
-                                                   const char *username)
-{
-  g_return_if_fail (EPHY_IS_WEB_EXTENSION_PROXY (web_extension));
-
-  g_signal_emit (web_extension, signals[FORM_AUTH_DATA_SAVE_REQUESTED], 0,
-                 request_id, page_id, hostname, username);
-}
-
 void
 ephy_web_extension_proxy_form_auth_data_save_confirmation_response (EphyWebExtensionProxy *web_extension,
                                                                     guint request_id,
@@ -210,7 +176,9 @@ ephy_web_extension_proxy_form_auth_data_save_confirmation_response (EphyWebExten
                      "FormAuthDataSaveConfirmationResponse",
                      g_variant_new ("(ub)", request_id, response),
                      G_DBUS_CALL_FLAGS_NONE,
-                     -1, NULL, NULL, NULL);
+                     -1,
+                     web_extension->priv->cancellable,
+                     NULL, NULL);
 }
 
 static void
@@ -250,7 +218,7 @@ ephy_web_extension_proxy_web_page_has_modified_forms (EphyWebExtensionProxy *web
                        g_variant_new ("(t)", page_id),
                        G_DBUS_CALL_FLAGS_NONE,
                        -1,
-                       cancellable,
+                       web_extension->priv->cancellable,
                        (GAsyncReadyCallback)has_modified_forms_cb,
                        g_object_ref (task));
   } else {
@@ -307,7 +275,7 @@ ephy_web_extension_proxy_get_best_web_app_icon (EphyWebExtensionProxy *web_exten
                        g_variant_new("(ts)", page_id, base_uri),
                        G_DBUS_CALL_FLAGS_NONE,
                        -1,
-                       cancellable,
+                       web_extension->priv->cancellable,
                        (GAsyncReadyCallback)get_best_web_app_icon_cb,
                        g_object_ref (task));
   } else {
@@ -380,7 +348,7 @@ ephy_web_extension_proxy_get_web_app_title (EphyWebExtensionProxy *web_extension
                        g_variant_new("(t)", page_id),
                        G_DBUS_CALL_FLAGS_NONE,
                        -1,
-                       NULL,
+                       web_extension->priv->cancellable,
                        (GAsyncReadyCallback)get_web_app_title_cb,
                        g_object_ref (task));
   } else {
@@ -421,7 +389,9 @@ ephy_web_extension_proxy_history_set_urls (EphyWebExtensionProxy *web_extension,
                      "HistorySetURLs",
                      g_variant_new ("(@a(ss))", g_variant_builder_end (&builder)),
                      G_DBUS_CALL_FLAGS_NONE,
-                     -1, NULL, NULL, NULL);
+                     -1,
+                     web_extension->priv->cancellable,
+                     NULL, NULL);
 }
 
 void
@@ -436,7 +406,9 @@ ephy_web_extension_proxy_history_set_url_thumbnail (EphyWebExtensionProxy *web_e
                      "HistorySetURLThumbnail",
                      g_variant_new ("(ss)", url, path),
                      G_DBUS_CALL_FLAGS_NONE,
-                     -1, NULL, NULL, NULL);
+                     -1,
+                     web_extension->priv->cancellable,
+                     NULL, NULL);
 }
 
 void
@@ -451,7 +423,9 @@ ephy_web_extension_proxy_history_set_url_title (EphyWebExtensionProxy *web_exten
                      "HistorySetURLTitle",
                      g_variant_new ("(ss)", url, title),
                      G_DBUS_CALL_FLAGS_NONE,
-                     -1, NULL, NULL, NULL);
+                     -1,
+                     web_extension->priv->cancellable,
+                     NULL, NULL);
 }
 
 void
@@ -465,7 +439,9 @@ ephy_web_extension_proxy_history_delete_url (EphyWebExtensionProxy *web_extensio
                      "HistoryDeleteURL",
                      g_variant_new ("(s)", url),
                      G_DBUS_CALL_FLAGS_NONE,
-                     -1, NULL, NULL, NULL);
+                     -1,
+                     web_extension->priv->cancellable,
+                     NULL, NULL);
 }
 
 void
@@ -479,7 +455,9 @@ ephy_web_extension_proxy_history_delete_host (EphyWebExtensionProxy *web_extensi
                      "HistoryDeleteHost",
                      g_variant_new ("(s)", host),
                      G_DBUS_CALL_FLAGS_NONE,
-                     -1, NULL, NULL, NULL);
+                     -1,
+                     web_extension->priv->cancellable,
+                     NULL, NULL);
 }
 
 void
@@ -492,5 +470,7 @@ ephy_web_extension_proxy_history_clear (EphyWebExtensionProxy *web_extension)
                      "HistoryClear",
                      NULL,
                      G_DBUS_CALL_FLAGS_NONE,
-                     -1, NULL, NULL, NULL);
+                     -1,
+                     web_extension->priv->cancellable,
+                     NULL, NULL);
 }
