@@ -109,7 +109,7 @@ struct _EphyWebViewPrivate {
   GTlsCertificateFlags tls_errors;
 
   gboolean loading_tls_error_page;
-  char *tls_error_page_host;
+  char *tls_error_failing_uri;
 
   /* Web Extension */
   EphyWebExtensionProxy *web_extension;
@@ -674,7 +674,7 @@ form_auth_data_save_confirmation_response (GtkInfoBar *info_bar,
 }
 
 static void
-form_auth_data_save_requested (EphyWebExtensionProxy *web_extension,
+form_auth_data_save_requested (EphyEmbedShell *shell,
                                guint request_id,
                                guint64 page_id,
                                const char *hostname,
@@ -699,22 +699,25 @@ form_auth_data_save_requested (EphyWebExtensionProxy *web_extension,
 }
 
 static void
-allow_tls_certificate_cb (EphyWebExtensionProxy *shell,
+allow_tls_certificate_cb (EphyEmbedShell *shell,
                           guint64 page_id,
                           EphyWebView *web_view)
 {
   EphyWebViewPrivate *priv = web_view->priv;
+  SoupURI *uri;
 
   if (webkit_web_view_get_page_id (WEBKIT_WEB_VIEW (web_view)) != page_id)
     return;
 
   g_return_if_fail (G_IS_TLS_CERTIFICATE (priv->certificate));
-  g_return_if_fail (priv->tls_error_page_host != NULL);
+  g_return_if_fail (priv->tls_error_failing_uri != NULL);
 
-  webkit_web_context_allow_tls_certificate_for_host (webkit_web_context_get_default (),
+  uri = soup_uri_new (priv->tls_error_failing_uri);
+  webkit_web_context_allow_tls_certificate_for_host (ephy_embed_shell_get_web_context (shell),
                                                      priv->certificate,
-                                                     priv->tls_error_page_host);
+                                                     uri->host);
   ephy_web_view_load_url (web_view, ephy_web_view_get_address (web_view));
+  soup_uri_free (uri);
 }
 
 static void
@@ -731,11 +734,11 @@ page_created_cb (EphyEmbedShell *shell,
   priv->web_extension = web_extension;
   g_object_add_weak_pointer (G_OBJECT (priv->web_extension), (gpointer *)&priv->web_extension);
 
-  g_signal_connect_object (priv->web_extension, "form-auth-data-save-requested",
+  g_signal_connect_object (shell, "form-auth-data-save-requested",
                            G_CALLBACK (form_auth_data_save_requested),
                            web_view, 0);
 
-  g_signal_connect_object (priv->web_extension, "allow-tls-certificate",
+  g_signal_connect_object (shell, "allow-tls-certificate",
                            G_CALLBACK (allow_tls_certificate_cb),
                            web_view, 0);
 }
@@ -779,7 +782,7 @@ ephy_web_view_finalize (GObject *object)
   g_free (priv->typed_address);
   g_free (priv->link_message);
   g_free (priv->loading_message);
-  g_free (priv->tls_error_page_host);
+  g_free (priv->tls_error_failing_uri);
 
   G_OBJECT_CLASS (ephy_web_view_parent_class)->finalize (object);
 }
@@ -1609,7 +1612,7 @@ load_changed_cb (WebKitWebView *web_view,
       priv->loading_tls_error_page = FALSE;
     } else {
       g_clear_object (&priv->certificate);
-      g_clear_pointer (&priv->tls_error_page_host, g_free);
+      g_clear_pointer (&priv->tls_error_failing_uri, g_free);
 
       if (webkit_web_view_get_tls_info (web_view, &priv->certificate, &priv->tls_errors)) {
         g_object_ref (priv->certificate);
@@ -1728,37 +1731,37 @@ detailed_message_from_tls_errors (GTlsCertificateFlags tls_errors)
 
   if (tls_errors & G_TLS_CERTIFICATE_BAD_IDENTITY) {
     /* Possible error message when a site presents a bad certificate. */
-    g_ptr_array_add (errors, _("This web site presented identification that belongs to a different web site."));
+    g_ptr_array_add (errors, _("This website presented identification that belongs to a different website."));
   }
 
   if (tls_errors & G_TLS_CERTIFICATE_EXPIRED) {
     /* Possible error message when a site presents a bad certificate. */
-    g_ptr_array_add (errors, _("This web site’s identification is too old to trust. Check the date on your computer’s calendar."));
+    g_ptr_array_add (errors, _("This website’s identification is too old to trust. Check the date on your computer’s calendar."));
   }
 
   if (tls_errors & G_TLS_CERTIFICATE_UNKNOWN_CA) {
     /* Possible error message when a site presents a bad certificate. */
-    g_ptr_array_add (errors, _("This web site’s identification was not issued by a trusted organization."));
+    g_ptr_array_add (errors, _("This website’s identification was not issued by a trusted organization."));
   }
 
   if (tls_errors & G_TLS_CERTIFICATE_GENERIC_ERROR) {
     /* Possible error message when a site presents a bad certificate. */
-    g_ptr_array_add (errors, _("This web site’s identification could not be processed. It may be corrupted."));
+    g_ptr_array_add (errors, _("This website’s identification could not be processed. It may be corrupted."));
   }
 
   if (tls_errors & G_TLS_CERTIFICATE_REVOKED) {
     /* Possible error message when a site presents a bad certificate. */
-    g_ptr_array_add (errors, _("This web site’s identification has been revoked by the trusted organization that issued it."));
+    g_ptr_array_add (errors, _("This website’s identification has been revoked by the trusted organization that issued it."));
   }
 
   if (tls_errors & G_TLS_CERTIFICATE_INSECURE) {
     /* Possible error message when a site presents a bad certificate. */
-    g_ptr_array_add (errors, _("This web site’s identification cannot be trusted because it uses very weak encryption."));
+    g_ptr_array_add (errors, _("This website’s identification cannot be trusted because it uses very weak encryption."));
   }
 
   if (tls_errors & G_TLS_CERTIFICATE_NOT_ACTIVATED) {
     /* Possible error message when a site presents a bad certificate. */
-    g_ptr_array_add (errors, _("This web site’s identification is only valid for future dates. Check the date on your computer’s calendar."));
+    g_ptr_array_add (errors, _("This website’s identification is only valid for future dates. Check the date on your computer’s calendar."));
   }
 
   if (errors->len == 1) {
@@ -1799,15 +1802,15 @@ get_tls_error_page_message (EphyWebView *view, const char *hostname)
   msg = g_strdup_printf ("<p>%s</p><p>%s</p><p>%s</p><p>%s <strong>%s</strong></p>",
                          warning,
                          /* Message when a site's TLS certificate is invalid. */
-                         _("When you try to connect securely, web sites present "
+                         _("When you try to connect securely, websites present "
                            "identification to prove that your connection has not been "
                            "maliciously intercepted. There is something wrong with "
-                           "this site’s identification:"),
+                           "this website’s identification:"),
                          details,
                          /* Message when a site's TLS certificate is invalid. */
                          _("A third party may have hijacked your connection. You should "
                            "continue only if you know there is a good reason why this "
-                           "site does not use trusted identification."),
+                           "website does not use trusted identification."),
                          /* Good advice from Firefox; displays when a site's TLS certificate is invalid. */
                          _("Legitimate banks, stores, and other public sites will "
                            "not ask you to do this."));
@@ -1905,7 +1908,8 @@ ephy_web_view_load_error_page (EphyWebView *view,
       /* Button on error page when a website's TLS certificate is invalid. */
       button_label = g_strdup (_("Load Anyway"));
       custom_class = "tls-error";
-      load_anyway_js = g_strdup ("EpiphanyTLSCertificateErrorPage.allowException();");
+      load_anyway_js = g_strdup_printf ("window.webkit.messageHandlers.tlsErrorPage.postMessage(%"G_GUINT64_FORMAT");",
+                                        webkit_web_view_get_page_id (WEBKIT_WEB_VIEW (view)));
       break;
     default:
       return;
@@ -2011,24 +2015,23 @@ load_failed_cb (WebKitWebView *web_view,
 
 static gboolean
 load_failed_with_tls_error_cb (WebKitWebView *web_view,
+                               const char *uri,
                                GTlsCertificate *certificate,
                                GTlsCertificateFlags errors,
-                               gchar *host,
                                gpointer user_data)
 {
   EphyWebView *view = EPHY_WEB_VIEW (web_view);
   EphyWebViewPrivate *priv = view->priv;
 
   g_clear_object (&priv->certificate);
-  g_clear_pointer (&priv->tls_error_page_host, g_free);
+  g_clear_pointer (&priv->tls_error_failing_uri, g_free);
 
   priv->loading_tls_error_page = TRUE;
   priv->certificate = g_object_ref (certificate);
   priv->tls_errors = errors;
-  priv->tls_error_page_host = g_strdup (host);
+  priv->tls_error_failing_uri = g_strdup (uri);
   ephy_web_view_set_security_level (EPHY_WEB_VIEW (web_view), EPHY_SECURITY_LEVEL_BROKEN_SECURITY);
-  ephy_web_view_load_error_page (EPHY_WEB_VIEW (web_view),
-                                 webkit_web_view_get_uri (web_view),
+  ephy_web_view_load_error_page (EPHY_WEB_VIEW (web_view), uri,
                                  EPHY_WEB_VIEW_ERROR_INVALID_TLS_CERTIFICATE, NULL);
 
   return TRUE;
@@ -2177,8 +2180,11 @@ ephy_web_view_init (EphyWebView *web_view)
 GtkWidget *
 ephy_web_view_new (void)
 {
+  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
+
   return g_object_new (EPHY_TYPE_WEB_VIEW,
-                       "user-content-manager", ephy_embed_shell_get_user_content_manager (ephy_embed_shell_get_default ()),
+                       "web-context", ephy_embed_shell_get_web_context (shell),
+                       "user-content-manager", ephy_embed_shell_get_user_content_manager (shell),
                        "settings", ephy_embed_prefs_get_settings (),
                        NULL);
 }
